'use strict';

var view = require('@codemirror/view');
var language = require('@codemirror/language');
var highlight = require('@lezer/highlight');

const base00 = '#212121', base01 = '#505d64', base02 = '#606f7a', base03 = '#707d8b', base04 = '#a0a4ae', base05 = '#bdbdbd', base06 = '#e0e0e0', base07 = '#fdf6e3', base_red = '#ff5f52', base_deeporange = '#ff6e40', base_pink = '#fa5788', base_yellow = '#facf4e', base_orange = '#ffad42', base_cyan = '#56c8d8', base_indigo = '#7186f0', base_purple = '#cf6edf', base_green = '#6abf69', base_lightgreen = '#99d066', base_teal = '#4ebaaa';
const invalid = base_red, darkBackground = '#202325', highlightBackground = '#545b61', background = base00, tooltipBackground = base01, selection = base01, cursor = base04;
/// The editor theme styles for Material Dark.
const materialDarkTheme = view.EditorView.theme({
    '&': {
        color: base05,
        backgroundColor: background
    },
    '.cm-content': {
        caretColor: cursor
    },
    '.cm-cursor, .cm-dropCursor': { borderLeftColor: cursor },
    '&.cm-focused > .cm-scroller > .cm-selectionLayer .cm-selectionBackground, .cm-selectionBackground, .cm-content ::selection': { backgroundColor: selection },
    '.cm-panels': { backgroundColor: darkBackground, color: base03 },
    '.cm-panels.cm-panels-top': { borderBottom: '2px solid black' },
    '.cm-panels.cm-panels-bottom': { borderTop: '2px solid black' },
    '.cm-searchMatch': {
        outline: `1px solid ${base_yellow}`,
        backgroundColor: 'transparent'
    },
    '.cm-searchMatch.cm-searchMatch-selected': {
        backgroundColor: highlightBackground
    },
    '.cm-activeLine': { backgroundColor: highlightBackground },
    '.cm-selectionMatch': {
        backgroundColor: darkBackground,
        outline: `1px solid ${base_teal}`
    },
    '&.cm-focused .cm-matchingBracket': {
        color: base06,
        outline: `1px solid ${base_teal}`
    },
    '&.cm-focused .cm-nonmatchingBracket': {
        color: base_red
    },
    '.cm-gutters': {
        backgroundColor: base00,
        borderRight: '1px solid #4f5b66',
        color: base02
    },
    '.cm-activeLineGutter': {
        backgroundColor: highlightBackground,
        color: base07
    },
    '.cm-foldPlaceholder': {
        backgroundColor: 'transparent',
        border: 'none',
        color: '#ddd'
    },
    '.cm-tooltip': {
        border: 'none',
        backgroundColor: tooltipBackground
    },
    '.cm-tooltip .cm-tooltip-arrow:before': {
        borderTopColor: 'transparent',
        borderBottomColor: 'transparent'
    },
    '.cm-tooltip .cm-tooltip-arrow:after': {
        borderTopColor: tooltipBackground,
        borderBottomColor: tooltipBackground
    },
    '.cm-tooltip-autocomplete': {
        '& > ul > li[aria-selected]': {
            backgroundColor: highlightBackground,
            color: base03
        }
    }
}, { dark: true });
/// The highlighting style for code in the Material Dark theme.
const materialDarkHighlightStyle = language.HighlightStyle.define([
    { tag: highlight.tags.keyword, color: base_purple },
    {
        tag: [highlight.tags.name, highlight.tags.deleted, highlight.tags.character, highlight.tags.macroName],
        color: base_cyan
    },
    { tag: [highlight.tags.propertyName], color: base_yellow },
    { tag: [highlight.tags.variableName], color: base05 },
    { tag: [highlight.tags.function(highlight.tags.variableName)], color: base_cyan },
    { tag: [highlight.tags.labelName], color: base_purple },
    {
        tag: [highlight.tags.color, highlight.tags.constant(highlight.tags.name), highlight.tags.standard(highlight.tags.name)],
        color: base_yellow
    },
    { tag: [highlight.tags.definition(highlight.tags.name), highlight.tags.separator], color: base_pink },
    { tag: [highlight.tags.brace], color: base_purple },
    {
        tag: [highlight.tags.annotation],
        color: invalid
    },
    {
        tag: [highlight.tags.number, highlight.tags.changed, highlight.tags.annotation, highlight.tags.modifier, highlight.tags.self, highlight.tags.namespace],
        color: base_orange
    },
    {
        tag: [highlight.tags.typeName, highlight.tags.className],
        color: base_orange
    },
    {
        tag: [highlight.tags.operator, highlight.tags.operatorKeyword],
        color: base_indigo
    },
    {
        tag: [highlight.tags.tagName],
        color: base_deeporange
    },
    {
        tag: [highlight.tags.squareBracket],
        color: base_red
    },
    {
        tag: [highlight.tags.angleBracket],
        color: base02
    },
    {
        tag: [highlight.tags.attributeName],
        color: base05
    },
    {
        tag: [highlight.tags.regexp],
        color: invalid
    },
    {
        tag: [highlight.tags.quote],
        color: base_green
    },
    { tag: [highlight.tags.string], color: base_lightgreen },
    {
        tag: highlight.tags.link,
        color: base_cyan,
        textDecoration: 'underline',
        textUnderlinePosition: 'under'
    },
    {
        tag: [highlight.tags.url, highlight.tags.escape, highlight.tags.special(highlight.tags.string)],
        color: base_yellow
    },
    { tag: [highlight.tags.meta], color: base03 },
    { tag: [highlight.tags.comment], color: base03, fontStyle: 'italic' },
    { tag: highlight.tags.monospace, color: base05 },
    { tag: highlight.tags.strong, fontWeight: 'bold', color: base_red },
    { tag: highlight.tags.emphasis, fontStyle: 'italic', color: base_lightgreen },
    { tag: highlight.tags.strikethrough, textDecoration: 'line-through' },
    { tag: highlight.tags.heading, fontWeight: 'bold', color: base_yellow },
    { tag: highlight.tags.heading1, fontWeight: 'bold', color: base_yellow },
    {
        tag: [highlight.tags.heading2, highlight.tags.heading3, highlight.tags.heading4],
        fontWeight: 'bold',
        color: base_yellow
    },
    {
        tag: [highlight.tags.heading5, highlight.tags.heading6],
        color: base_yellow
    },
    { tag: [highlight.tags.atom, highlight.tags.bool, highlight.tags.special(highlight.tags.variableName)], color: base_cyan },
    {
        tag: [highlight.tags.processingInstruction, highlight.tags.inserted],
        color: base_red
    },
    {
        tag: [highlight.tags.contentSeparator],
        color: base_cyan
    },
    { tag: highlight.tags.invalid, color: base02, borderBottom: `1px dotted ${base_red}` }
]);
/// Extension to enable the Material Dark theme (both the editor theme and
/// the highlight style).
const materialDark = [
    materialDarkTheme,
    language.syntaxHighlighting(materialDarkHighlightStyle)
];

exports.materialDark = materialDark;
exports.materialDarkHighlightStyle = materialDarkHighlightStyle;
exports.materialDarkTheme = materialDarkTheme;
